/******************************************************
STM8S003K3 16MHz, IAR1.4+STM8S_StdPeriph_Lib_V2.1.0
******************************************************/
#include "stm8s.h"
#include "iostm003.c"
#include "bitset.c"

#define A       PD_ODR_ODR7
#define B       PD_ODR_ODR6
#define C       PD_ODR_ODR5
#define D       PD_ODR_ODR4
#define E       PD_ODR_ODR3
#define F       PD_ODR_ODR2
#define G       PD_ODR_ODR1
#define DP      PC_ODR_ODR7
#define N0      PC_ODR_ODR6
#define N1      PC_ODR_ODR5
#define N2      PC_ODR_ODR4
#define N3      PC_ODR_ODR3

#define button0      PB_IDR_IDR7

#define IR2110_off   PE_ODR_ODR5
#define DCAC_on      PB_ODR_ODR0

#define R_upr 12000 //Ohms
#define R_lwr 5100   //Ohms
#define Ub_max_adc ((500UL*(R_upr+R_lwr))/R_lwr) //=U_max*100, 16.76V*100

#define Rhv_upr 680000 //Ohms
#define Rhv_lwr 10000   //Ohms
#define UHV_max_adc ((500UL*(Rhv_upr+Rhv_lwr))/Rhv_lwr) //=U_max*100, 345V*100

  #define uptrs	340UL
  #define svl	   320UL
  #define ltrs	   300UL

   const u16 arrv=0x01ff;

   const u16 x0 = (ltrs*0x3ff*100)/(UHV_max_adc);
   const u16 x1 = (svl*0x3ff*100)/(UHV_max_adc);
   const u16 x2 = (uptrs*0x3ff*100)/(UHV_max_adc);

enum btn_states {b_prs, b_rls, b_hold} btn0=b_rls;
enum dmodes {normal,started,lobat,sdn,sleep,ocp,ovp,ovlp} d_mode=started;

volatile u16 tim0,tim1,tim2,tim3;
volatile u16 adc2val, adc3val;
u8 bat_ok;
volatile u8 hv_ok=0, hv_en=0, ovrld=0, ovl_fg=0;;

u8 display_buffer[4]={0,0,0,1};
u8 dppos=0;

inline static void gpio_cfg() //ch def at resort
{
  	PA_DDR=PB_DDR=PC_DDR=PD_DDR=PE_DDR=PF_DDR=0xff; //conf as output
	PA_CR1=PB_CR1=PC_CR1=PD_CR1=PE_CR1=PF_CR1=0xff; //push-pull|pullup
  	//PA_CR2=PB_CR2=PC_CR2=PD_CR2=PE_CR2=PF_CR2=0;  //speed 2MHz|int disabled

	PB_DDR_DDR2 = PB_DDR_DDR3 = 0;
   PB_CR1_C12  = PB_CR1_C13  = 0; //pullup off

	PD_DDR_DDR0 = 0; //BKIN
   PD_CR1_C10  = 1;

}
inline static void clk_cfg()
{
  	CLK_DeInit();
	CLK->CKDIVR = 0;     //16MHz
	CLK->PCKENR1 = 0x80; //only tim1 clk enabled
}

inline static void t1_cfg()
{

  {
   #define ccrdv 16

	TIM1->ARRH = arrv >> 8; //arrv=0x01ff
  	TIM1->ARRL = arrv&0xff;

   TIM1->CCR1H=0; //(arrv/ccrdv)>>8;
   TIM1->CCR1L=(arrv/ccrdv)&0xff;
   TIM1->CCR2H=1; //(arrv-arrv/ccrdv)>>8;
   TIM1->CCR2L=(arrv-(arrv/ccrdv))&0xff;
  }


	TIM1->CR1 = TIM1_COUNTERMODE_CENTERALIGNED1|TIM1_CR1_ARPE|TIM1_CR1_URS;
	TIM1->CR2 = TIM1_CR2_CCPC;
   TIM1->RCR = 1;

   TIM1->CCMR1|= TIM1_OCMODE_PWM1|TIM1_CCMR_OCxPE;
	TIM1->CCMR2|= TIM1_OCMODE_PWM2|TIM1_CCMR_OCxPE;
   TIM1->CCER1|= TIM1_CCER1_CC1E;
	TIM1->CCER1|= TIM1_CCER1_CC2E;
	TIM1->EGR  |= TIM1_EGR_COMG;  //update CCER reg with COM event
   TIM1->IER   = TIM1_IER_UIE;   //update interrupt enable

	TIM1->BKR  = TIM1_BKR_MOE|TIM1_BKR_OSSR|TIM1_BKR_OSSI|TIM1_BKR_BKE;
	TIM1->OISR = 0;
	TIM1->CR1 |= TIM1_CR1_CEN;
 	TIM1->BKR |= TIM1_LOCKLEVEL_3;

}

inline static void adc_cfg()
{
	ADC1->CR1 = ADC1_PRESSEL_FCPU_D18;
	ADC1->CR2 = ADC1_ALIGN_RIGHT;
   ADC1->CSR = 1; //channel sel

	ADC1->TDRL = binary(00001100); //smtt trg on adc pins is off
	ADC1->CR1 |= ADC1_CR1_ADON;
}

void convert_data (u16 d)
{
u8 k;
  for (k=0;k<4;k++)
  {
   display_buffer[k]=d%10;
   d/=10;
  }
}

void display_data ()
{
  static int display_counter;

  (display_counter)?(display_counter--):(display_counter = 3);
  N0=N1=N2=N3=0; DP=1;

  switch (display_buffer[display_counter])
 {

  case 0:  A=0; B=0; C=0; D=0; E=0; F=0; G=1; break;
  case 1:  A=1; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 2:  A=0; B=0; C=1; D=0; E=0; F=1; G=0; break;
  case 3:  A=0; B=0; C=0; D=0; E=1; F=1; G=0; break;
  case 4:  A=1; B=0; C=0; D=1; E=1; F=0; G=0; break;
  case 5:  A=0; B=1; C=0; D=0; E=1; F=0; G=0; break;
  case 6:  A=0; B=1; C=0; D=0; E=0; F=0; G=0; break;
  case 7:  A=0; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 8:  A=0; B=0; C=0; D=0; E=0; F=0; G=0; break;
  case 9:  A=0; B=0; C=0; D=0; E=1; F=0; G=0; break;
  case 10: A=1; B=1; C=1; D=1; E=1; F=1; G=1; break;
  case 11: A=1; B=1; C=0; D=0; E=0; F=1; G=1; break; //u
  case 12: A=1; B=1; C=1; D=0; E=0; F=1; G=0; break; //c
  case 13: A=1; B=1; C=0; D=0; E=0; F=0; G=0; break; //b
  case 14: A=1; B=1; C=1; D=0; E=0; F=0; G=0; break; //t
  case 15: A=1; B=1; C=1; D=0; E=0; F=1; G=1; break; //l
  case 16: A=1; B=1; C=0; D=0; E=0; F=1; G=0; break; //o
  case 17: A=0; B=0; C=0; D=1; E=0; F=0; G=0; break; //A
  case 18: A=1; B=0; C=0; D=1; E=0; F=0; G=0; break; //H
  case 19: A=0; B=1; C=1; D=0; E=0; F=0; G=1; break; //C
  case 20: A=0; B=0; C=1; D=1; E=0; F=0; G=0; break; //P
  case 21: A=0; B=1; C=1; D=0; E=0; F=0; G=0; break; //E
  case 22: A=1; B=0; C=0; D=0; E=0; F=0; G=1; break; //U
  case 23: A=0; B=1; C=1; D=1; E=0; F=0; G=0; break; //F
  case 24: A=1; B=1; C=1; D=0; E=0; F=0; G=1; break; //L
  case 25: A=1; B=1; C=1; D=1; E=0; F=1; G=0; break; //r
  case 26: A=1; B=1; C=1; D=0; E=1; F=1; G=1; break; //_
  case 27: A=1; B=0; C=0; D=0; E=0; F=1; G=0; break; //d
  case 28: A=1; B=1; C=0; D=1; E=0; F=1; G=0; break; //n
  default: A=1; B=1; C=1; D=1; E=1; F=1; G=0; break; //-
 }

   switch (display_counter)
 {
  case 0: N0=1; if(dppos == 0) {DP=0;} break;
  case 1: N1=1; if(dppos == 1) {DP=0;} break;
  case 2: N2=1; if(dppos == 2) {DP=0;} break;
  case 3: if(display_buffer[3]){N3=1;} if(dppos == 3) {DP=0;} break;
  default: DP=1; break;
 }
 }

void delay(volatile uint16_t nCount)
{
    while (nCount)
    {
        nCount--;
        IWDG_ReloadCounter();
    }
}

void dev_mgmt();void bat_detect();void hv_detect();void dbv(u8 adc_cnl);
void pwm_null();

INTERRUPT_HANDLER(TIM1_UPD_OVF_TRG_BRK_IRQHandler, 11)
{
  {//adc csn
  #define rcntdef 8
  static u8 rcnt3=rcntdef, rcnt2=rcntdef;
  static u16 tmp3=0, tmp2=0;
  u8  adch = 0;
  u8  adcl = 0;

  if(!rcnt2)
  {
    if(rcnt3) //vbat
    {
      rcnt3--;
      adcl = ADC1->DRL;
      adch = ADC1->DRH;
      tmp3+=(u16)(adcl|(u16)(adch<<8));
    }
    else
    {
      rcnt3=rcntdef; adc3val=tmp3/(rcntdef); tmp3=0;
    }
    ADC1->CSR &= ~ADC1_CSR_CH;
    ADC1->CSR |= 2;
    rcnt2=rcntdef;
  }
  else
  {
      rcnt2--;
      adcl = ADC1->DRL;
      adch = ADC1->DRH;
      tmp2+=(u16)(adcl|(u16)(adch<<8));

    if(!rcnt2)
    {
      adc2val= tmp2/(rcntdef); tmp2=0;
      ADC1->CSR &= ~ADC1_CSR_CH;
      ADC1->CSR |= 3;
    }
  }
  }

   if(hv_en)
	{
     if(adc2val<x1){if(TIM1->CCR1L<0xff-5){TIM1->CCR1L++;}
                    else if(adc2val<x0){if(ovrld<0xff){ovrld++;}else{ovl_fg=1;}}}
     if(adc2val>x2){if(TIM1->CCR1L>9){TIM1->CCR1L-=10;}else{TIM1->CCR1L=0;}}
     else if(adc2val>x1){if(TIM1->CCR1L){TIM1->CCR1L--;} if(ovrld){ovrld--;}else{ovl_fg=0;}}
     else if(adc2val==x1){if(ovrld){ovrld--;}else{ovl_fg=0;}}

     TIM1->CCR2L = 0xff-TIM1->CCR1L;
	}



    if(tim0)tim0--;
    if(tim1)tim1--;
    if(tim2)tim2--;
    if(tim3)tim3--;

   ADC1->CR1 |= ADC1_CR1_ADON;
   TIM1->EGR |= TIM1_EGR_COMG;
	TIM1->SR1 = 0;	//clear interrupts flags
}

void main()
{

gpio_cfg();
clk_cfg();
adc_cfg();
t1_cfg();

ADC1_StartConversion();


asm("rim");


  while(1)
  {
		dev_mgmt();
      display_data ();



  }
}

void assert_failed(uint8_t* file, uint32_t line)
{

}

void bat_detect()
{
  u16 tmp;

  tmp=((unsigned long)adc3val*Ub_max_adc)/(0x3ff*10);

   if(tmp>108){bat_ok=1;tim1=0xffff;}
   else if ((tmp<90)||(!tim1)){bat_ok=0;}
}
void hv_detect()
{
   if((adc2val>x0)&&(adc2val<x2)){hv_ok=1; tim0=0x1fff;}
   else if ((!tim0)||(ovl_fg))  {hv_ok=0;}
}

void dev_mgmt()
{

 switch (d_mode)
 {
  case started:
  {
   hv_en=0; DCAC_on=0; IR2110_off=0;

	while (!bat_ok)
   {
    dbv(3);
    bat_detect();
	 display_data ();
   }

	while (adc2val<x1)
	{
	  dbv(2);
     bat_detect();
	  display_data ();

	 if(bat_ok)
	 {
         if(!tim3)
          {
            tim3=0x7d;
            if(TIM1->CCR1L<0xff-5)
				{asm("sim");TIM1->CCR1L++;TIM1->CCR2L = 0xff-TIM1->CCR1L;asm("rim");}				
            else 
				{
				 d_mode=ovlp;  pwm_null();
				 tim3=0xffff;
				 //ovlP
				 display_buffer[3]=16;
				 display_buffer[2]=11;
				 display_buffer[1]=15;
				 display_buffer[0]=20;
				 dppos=5;
				 while(tim3){display_data ();}
				 break;
				}

          }
    }
	 else 
	 {
	  d_mode=lobat; pwm_null();
	  tim3=0xffff;
     //Lo.bt
	  display_buffer[3]=15;
	  display_buffer[2]=16;
	  display_buffer[1]=13;
	  display_buffer[0]=14;
	  dppos=2;
	  while(tim3){display_data ();} 		
	  break;
	 }

	 if(!(TIM1->BKR&TIM1_BKR_MOE)) 
	 {
	  d_mode = ocp; pwm_null();
	  tim3=0xffff;
     //OCP
	  display_buffer[3]=0;
	  display_buffer[2]=0;
	  display_buffer[1]=19;
	  display_buffer[0]=20;
	  dppos=5;
	  while(tim3){display_data ();} 
	  break;
	 }
	}

	if(d_mode==started)
	{
	hv_en = 1;
   DCAC_on=1;
	d_mode = normal;
   ovrld=ovl_fg=0;
	}
  }
  break;

  case normal:
  {
   static u8 trgn=0;
	
	if(!trgn)
	{
	 trgn=1;
	 tim3=0xffff;
	 display_buffer[3]=28;
	 display_buffer[2]=16;
	 display_buffer[1]=25;
	 display_buffer[0]=15;
	 dppos=5;//norl
	}
	else if(!tim3){dbv(3);}
	
	bat_detect();
	hv_detect();

	if(!bat_ok)       
	{
	  d_mode=lobat; pwm_null();trgn=0;
	  tim3=0xffff;
     //Lo.bt
	  display_buffer[3]=15;
	  display_buffer[2]=16;
	  display_buffer[1]=13;
	  display_buffer[0]=14;
	  dppos=2;
	  while(tim3){display_data ();}	
	}
	
	else if(!hv_ok)   
	{
	  d_mode=ovp;pwm_null();trgn=0;
	  tim3=0xffff;
     //OUP
	  display_buffer[3]=0;
	  display_buffer[2]=0;
	  display_buffer[1]=22;
	  display_buffer[0]=20;
	  dppos=5;
	  while(tim3){display_data ();}	
	}
	
   else if(ovl_fg)   
	{
	  d_mode=ovlp;pwm_null();trgn=0;
	  tim3=0xffff;
     //ovlP
	  display_buffer[3]=16;
	  display_buffer[2]=11;
	  display_buffer[1]=15;
	  display_buffer[0]=20;
	  dppos=5;
	  while(tim3){display_data ();}	
	}
	
	if(!(TIM1->BKR&TIM1_BKR_MOE))
	{
	  d_mode = ocp; pwm_null();trgn=0;
	  tim3=0xffff;
     //OCP
	  display_buffer[3]=0;
	  display_buffer[2]=0;
	  display_buffer[1]=19;
	  display_buffer[0]=20;
	  dppos=5;
	  while(tim3){display_data ();}	
	}

  }
  break;

  case lobat:
  {
	static u8 trg = 0, cnt;
	u16 tmp = 0;

	if(!trg)
   {
     trg=1;cnt = 0xff/2;
   }

	if(!tim3)
	{
     tmp=((unsigned long)adc3val*Ub_max_adc)/(0x3ff*10);

	  tim3=0x01ee;

     if(tmp>115){cnt++;}
     else if(cnt){cnt--;}

	  if (cnt>0xff-1){trg=0; d_mode=started;}
	  else if(!cnt) {trg=0; d_mode = sdn;}
	}
  }
  break;

  case sdn:
  {
   d_mode = sleep;
  }
  break;

  case sleep:
  {
	  tim3=0xffff;
     //HALt
	  display_buffer[3]=18;
	  display_buffer[2]=17;
	  display_buffer[1]=24;
	  display_buffer[0]=14;
	  dppos=5;
	  while(tim3){display_data ();}	

	  asm("sim");
	  PA_ODR=PB_ODR=PC_ODR=PD_ODR=PE_ODR=PF_ODR=0;
	  halt();
  }
  break;

  case ocp:
  {

	TIM1->BKR |= TIM1_BKR_MOE;
	d_mode = started;

  }
  break;

  case ovp:
  {
   d_mode = started;
  }
  break;

  case ovlp:
  {
  d_mode = started;
  }
  break;

  default: break;
 }
}

void dbv(u8 adc_cnl)
{
  if(!tim2)
  {
    tim2=0x1f40;
    if(adc_cnl==3)
	 {convert_data (((unsigned long)adc3val*Ub_max_adc)/(0x3ff*10));dppos=1;}
    else
    {convert_data (((unsigned long)adc2val*UHV_max_adc)/(0x3ffUL*100));dppos=5;}
  }
}
void pwm_null()
{
	hv_en=0; DCAC_on=0; IR2110_off=1;
   asm("sim");
	TIM1->CCR1L=0;TIM1->CCR2L=0xff;
	asm("rim");
}